@@ -9,36 +9,43 @@
     instPkgs[too_new, c("Version", "LibPath"), drop=FALSE]
 }
 
-.valid <-
-    function(pkgs = installed.packages(lib.loc, priority=priority),
+.valid_out_of_date_pkgs <-
+    function(pkgs = installed.packages(lib.loc, priority=priority), repos,
              lib.loc=NULL, priority="NA", type=getOption("pkgType"),
-             filters=NULL, ..., checkBuilt, site_repository,
-             version=BiocManager::version())
+             filters=NULL, ..., checkBuilt, site_repository)
 {
-    version <- .version_validate(version)
-    repos <- .repositories(site_repository, version = version)
-    repos <- .repositories_filter(repos)
-
     contribUrl <- contrib.url(repos, type=type)
 
     available <- out_of_date <- too_new <- character()
     result <- FALSE
-    if (length(repos)) {
-        available <- .inet_available.packages(
-            contribUrl, type=type, filters=filters
-        )
+    available <- .inet_available.packages(
+        contribUrl, type=type, filters=filters
+    )
+    out_of_date <- .inet_old.packages(
+        lib.loc, repos=repos, instPkgs=pkgs,
+        available=available, checkBuilt=checkBuilt, type=type
+    )
+    list(
+        available = available,
+        out_of_date = out_of_date,
+        noRepos = !length(repos)
+    )
+}
 
-        out_of_date <- .inet_old.packages(
-            lib.loc, repos=repos, instPkgs=pkgs,
-            available=available, checkBuilt=checkBuilt, type=type
-        )
+.valid_out_of_date_filter <- function(out_of_date) {
+    out_of_date[rownames(out_of_date) != "BiocVersion", , drop = FALSE]
+}
 
-        too_new <- .valid_pkgs_too_new(pkgs, available)
+.valid_result <-
+    function(avail_out, pkgs = installed.packages(lib.loc, priority=priority),
+             lib.loc=NULL, priority="NA")
+{
+    too_new <- .valid_pkgs_too_new(pkgs, avail_out[["available"]])
+    out_of_date <- .valid_out_of_date_filter(avail_out[["out_of_date"]])
 
-        result <- !nrow(too_new) && is.null(out_of_date)
-    }
+    result <- !nrow(too_new) && (is.null(out_of_date) || !nrow(out_of_date))
 
-    if (!result) {
+    if (!result || avail_out[["noRepos"]]) {
         result <- structure(
             list(out_of_date = out_of_date, too_new = too_new),
             class="biocValid"
@@ -53,35 +60,35 @@
 #' nor too new) with the version of R and _Bioconductor_ in use.
 #'
 #' @details This function compares the version of installed packages
-#'     to the version of packages associated with the version of _R_
-#'     and _Bioconductor_ currently in use.
+#'   to the version of packages associated with the version of _R_
+#'   and _Bioconductor_ currently in use.
 #'
-#'     Packages are reported as 'out-of-date' if a more recent version
-#'     is available at the repositories specified by
-#'     `BiocManager::repositories()`.  Usually, `BiocManager::install()` is
-#'     sufficient to update packages to their most recent version.
+#'   Packages are reported as 'out-of-date' if a more recent version
+#'   is available at the repositories specified by
+#'   `BiocManager::repositories()`.  Usually, `BiocManager::install()` is
+#'   sufficient to update packages to their most recent version.
 #'
-#'     Packages are reported as 'too new' if the installed version is
-#'     more recent than the most recent available in the
-#'     `BiocManager::repositories()`. It is possible to down-grade by
-#'     re-installing a too new package "PkgA" with
-#'     `BiocManger::install("PkgA")`. It is important for the user to
-#'     understand how their installation became too new, and to avoid
-#'     this in the future.
+#'   Packages are reported as 'too new' if the installed version is
+#'   more recent than the most recent available in the
+#'   `BiocManager::repositories()`. It is possible to down-grade by
+#'   re-installing a too new package "PkgA" with
+#'   `BiocManger::install("PkgA")`. It is important for the user to
+#'   understand how their installation became too new, and to avoid
+#'   this in the future.
 #'
 #' @param pkgs A character() vector of package names for checking, or
-#'     a matrix as returned by `\link{installed.packages}`.
+#'     a matrix as returned by \code{\link{installed.packages}()}`.
 #' @param lib.loc A character() vector of library location(s) of
-#'     packages to be validated; see `\link{installed.packages}()`.
+#'     packages to be validated; see \code{\link{installed.packages}()}.
 #' @param priority character(1) Check validity of all, "base", or
-#'     "recommended" packages; see `\link{installed.packages}()`.
+#'     "recommended" packages; see \code{\link{installed.packages}()}.
 #' @param type character(1) The type of available package (e.g.,
 #'     binary, source) to check validity against; see
-#'     `\link{available.packages}()`.
+#'     \code{\link{available.packages}()}.
 #' @param filters character(1) Filter available packages to check
-#'     validity against; see `\link{available.packages}()`.
+#'     validity against; see \code{\link{available.packages}()}.
 #' @param \dots Additional arguments, passed to
-#'     `BiocManager::\link{install}()` when `fix=TRUE`.
+#'     \code{BiocManager::\link{install}()} when `fix=TRUE`.
 #' @param checkBuilt `logical(1)`. If `TRUE` a package built under an
 #'     earlier major.minor version of R (e.g., 3.4) is considered to
 #'     be old.
@@ -89,13 +96,17 @@
 #' @return `biocValid` list object with elements `too_new` and
 #'     `out_of_date` containing `data.frame`s with packages and their
 #'     installed locations that are too new or out-of-date for the
-#'     current version of _Bioconductor_.
+#'     current version of _Bioconductor_. When internet access
+#'     is unavailable, an empty 'biocValid' list is returned. If all
+#'     packages ('pkgs') are up to date, then TRUE is returned.
 #' @author Martin Morgan \email{martin.morgan@@roswellpark.org}
-#' @seealso `BiocManager::\link{install}()` to update installed
+#' @seealso \code{BiocManager::\link{install}()} to update installed
 #'     packages.
 #' @keywords environment
 #' @examples
-#' BiocManager::valid()
+#' if (interactive()) {
+#'   BiocManager::valid()
+#' }
 #' @md
 #' @export valid
 valid <-
@@ -107,6 +118,8 @@ valid <-
     stopifnot(
         is.logical(checkBuilt), length(checkBuilt) == 1L, !is.na(checkBuilt)
     )
+    site_repository <- .repositories_site_repository(site_repository)
+
     if (!is.matrix(pkgs)) {
         if (is.character(pkgs)) {
             pkgs <- installed.packages(pkgs, lib.loc=lib.loc)
@@ -117,10 +130,17 @@ valid <-
             )
         }
     }
-    result <- .valid(
-        pkgs, lib.loc, priority, type, filters, ...,
-        checkBuilt = checkBuilt, site_repository = site_repository
-    )
+    version <- .version_validate(version())
+    repos <- .repositories(site_repository, version = version)
+    repos <- .repositories_filter(repos)
+
+    vout <- .valid_out_of_date_pkgs(pkgs = pkgs, lib.loc = lib.loc,
+        repos = repos, priority = priority, type = type, filters=filters, ...,
+        checkBuilt = checkBuilt, site_repository = site_repository)
+
+    result <-
+        .valid_result(vout, pkgs = pkgs, lib.loc = lib.loc, priority = priority)
+
     if (!isTRUE(result)) {
         out_of_date <- result$out_of_date
         too_new <- result$too_new
@@ -158,7 +178,7 @@ print.biocValid <-
         return()
     }
 
-    fmt <-'  BiocManager::install(%s, update = TRUE, ask = FALSE)'
+    fmt <-'  BiocManager::install(%s, update = TRUE, ask = FALSE, force = TRUE)'
     if (n == 1L) {
         fmt <- sprintf(fmt, '"%s"')
     } else {
